#ifndef IMG_H
#define IMG_H

/***
 *** Image support module
 *** Allow easy interface to sgi imagelib
 ***
 *** Code done by Sebastien Roy (sebastien@research.nj.nec.com)
 ***
 ***/

/*
 *               Copyright (c) NEC Research Institute .
 *                          All Rights Reserved.
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to NEC a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions
 * made to the core functions of the software, including but not limited to
 * those affording compatibility with other hardware or software
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to NEC any
 * such changes, enhancements or extensions that they make and inform NEC
 * of noteworthy uses of this software.  Correspondence should be provided
 * to NEC at:
 *
 *                     Ingemar J. Cox
 *
 *                     NEC Research Institute
 *                     4 Independence Way
 *                     Princeton
 *                     NJ 08540
 *
 *                     phone:  609 951 2722
 *                     fax:  609 951 2482
 *                     email:  ingemar@research.nj.nec.com (Inet)
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND NEC RESEARCH INSTITUTE DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL NEC RESEARCH
 * INSTITUTE BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Software by Sebastien Roy (sebastien@research.nj.nec.com)
 * Last modified May 12, 1995
 */



/** Debugging assertions for interpolation **/
#define NDEBUG

#ifdef NDEBUG
#define ASS_IMG_XY(x,y,xs,ys)
#define ASS_IMG_P(p,xs,ys)
#else
#define ASS_IMG_XY(x,y,xs,ys)	assert(x>=0 && y>=0 && x<xs && y<ys)
#define ASS_IMG_P(p,xs,ys)	assert(p>=0 && p<(xs*ys))
#endif


/** Information about the images **/
typedef struct{
        int XSize,YSize;        /* size of image */
	int ZSize;		/* 1->Monochrome, 3->RGB */
	unsigned char *Data;	/* Image Data, row order */
				/* In RGB mode : RGB RGB RGB RGB ... */
				/* NULL -> No Image loaded */
        } imginfo;


/*** Routines ***/

void DumpImgInfo(imginfo *II);

/* init even without loading any image */
int InitImgInfo(imginfo *II,int xs,int ys);

/** Create a new image **/
int CreateImage(int XS,int YS,int ZS,imginfo *I);

void FreeImage(imginfo *I);

/** return 0 if ok, -1 if error **/
/** return the data in PData, and the size in P[XYZ]Size **/
/** ZSize=1 -> monochrome, ZSize=3 or 4 -> RGB or ARGB **/
/** If RGB : rgbrgbrgbrgb... **/
int LoadImage(char *Name,imginfo *I);

/*** Save in IRIS Format , If color, Data is RGB RGB RGB ... ***/
int SaveImage(char *Name,imginfo *I);
int SaveImageNoRLE(char *Name,imginfo *I);

/* Bi-linear Interpolation */
/* Return 1 if (x,y) are inside the image, 0 otherwise */
int ImgCheck(float x,float y,imginfo *I);

/* Bi-linear interpolation */
/* Use z=0 in monochrome case */
float InterpoleImg(float x,float y,int iz,imginfo *I);


/* Bi-Cubic Interpolation */
/* Return 1 if (x,y) are inside the image, 0 otherwise */
int ImgCheckCubic(float x,float y,imginfo *I);

/* Bi-Cubic interpolation */
/* Use z=0 in monochrome case */
float InterpoleImgCubic(float x,float y,int iz,imginfo *I);

#endif
